@@ -93,6 +93,7 @@ impl NetworkFrame {
 
     //Valid severities can be found here: https://www.postgresql.org/docs/current/protocol-error-fields.html
     //Valid error codes can be found here: https://www.postgresql.org/docs/current/errcodes-appendix.html
+    //TODO #26 need to figure out a better error method to show errors
     pub fn error_response(
         severity: PgErrorLevels,
         code: PgErrorCodes,
@@ -110,10 +111,7 @@ impl NetworkFrame {
         buffer.put_u8(b'\0');
         buffer.put_u8(b'\0');
 
-        NetworkFrame::new(
-            b'N', //Testing notifications
-            buffer.freeze(),
-        )
+        NetworkFrame::new(b'E', buffer.freeze())
     }
 }
 
