@@ -2,6 +2,8 @@ mod nullable;
 pub use nullable::Nullable;
 
 mod page_settings;
+pub use page_settings::MAX_FILE_HANDLE_COUNT;
+pub use page_settings::MAX_PAGE_CACHE;
 pub use page_settings::PAGES_PER_FILE;
 pub use page_settings::PAGE_SIZE;
 
@@ -11,5 +13,5 @@ pub use pg_error_codes::PgErrorCodes;
 mod pg_error_levels;
 pub use pg_error_levels::PgErrorLevels;
 
-mod table_definitions;
-pub use table_definitions::TableDefinitions;
+pub mod system_tables;
+pub use system_tables::SystemTables;
