@@ -1,7 +1,7 @@
+use crate::constants::SystemTables;
 use crate::engine::objects::types::BaseSqlTypes;
-use crate::engine::objects::SqlTuple;
+use crate::engine::objects::{ConstraintMapper, SqlTuple};
 
-use super::super::constants::TableDefinitions;
 use super::io::{ConstraintManager, ConstraintManagerError};
 use super::objects::types::SqlTypeDefinition;
 use super::objects::{ParseTree, Plan, PlannedStatement, SqlTupleError, Table};
@@ -17,7 +17,7 @@ use uuid::Uuid;
 
 //TODO way too many clones / Arc flipping. Unsure if I could make use of references better
 
-#[derive(Clone, Debug)]
+#[derive(Clone)]
 pub struct Executor {
     cons_man: ConstraintManager,
 }
@@ -47,9 +47,7 @@ impl Executor {
             Plan::FullTableScan(fts) => {
                 self.full_table_scan(tran_id, fts.src_table.clone(), fts.target_type.clone())
             }
-            Plan::ModifyTable(mt) => {
-                self.modify_table(tran_id, mt.table.clone(), mt.source.clone())
-            }
+            Plan::ModifyTable(mt) => self.modify_table(tran_id, &mt.table, mt.source.clone()),
             Plan::StaticData(sd) => self.static_data(sd.clone()),
         }
     }
@@ -98,16 +96,17 @@ impl Executor {
     fn modify_table(
         self,
         tran_id: TransactionId,
-        table: Arc<Table>,
+        table: &Arc<Table>,
         source: Arc<Plan>,
     ) -> Pin<Box<impl Stream<Item = Result<SqlTuple, ExecutorError>>>> {
         let vis = self.clone().cons_man;
+        let table = table.clone();
 
         let s = try_stream! {
             for await val in self.clone().execute_plans(tran_id, source) {
                 let unwrapped_val = val?;
                 vis.clone()
-                    .insert_row(tran_id, table.clone(), unwrapped_val.clone())
+                    .insert_row(tran_id, &table, unwrapped_val.clone())
                     .await?;
                 yield unwrapped_val;
             }
@@ -129,11 +128,11 @@ impl Executor {
 
     //Bypass planning since there isn't anything optimize
     pub async fn execute_utility(
-        &self,
+        &mut self,
         tran_id: TransactionId,
         parse_tree: ParseTree,
     ) -> Result<Vec<SqlTuple>, ExecutorError> {
-        let cm = self.cons_man.clone();
+        let mut cm = self.cons_man.clone();
 
         let create_table = match parse_tree {
             ParseTree::CreateTable(t) => t,
@@ -141,15 +140,17 @@ impl Executor {
         };
 
         let table_id = Uuid::new_v4();
-        let pg_class = TableDefinitions::PgClass.value();
+        let pg_class = SystemTables::PgClass.value();
         let table_row = SqlTuple(vec![
             Some(BaseSqlTypes::Uuid(table_id)),
             Some(BaseSqlTypes::Text(create_table.table_name.clone())),
         ]);
 
-        cm.insert_row(tran_id, pg_class, table_row).await?;
+        cm.insert_row(tran_id, &pg_class, table_row).await?;
 
-        let pg_attribute = TableDefinitions::PgAttribute.value();
+        let mut primary_key_cols = vec![];
+
+        let pg_attribute = SystemTables::PgAttribute.value();
         for i in 0..create_table.provided_columns.len() {
             let cm = self.cons_man.clone();
             let i_u32 = u32::try_from(i).map_err(ExecutorError::ConversionError)?;
@@ -166,9 +167,51 @@ impl Executor {
                 Some(BaseSqlTypes::Bool(create_table.provided_columns[i].null)),
             ]);
             cm.clone()
-                .insert_row(tran_id, pg_attribute.clone(), table_row)
+                .insert_row(tran_id, &pg_attribute, table_row)
+                .await?;
+
+            if create_table.provided_columns[i].primary_key {
+                primary_key_cols.push(BaseSqlTypes::Integer(i_u32));
+            }
+        }
+
+        if !primary_key_cols.is_empty() {
+            //We assume the the order that columns with primary key were defined are the order desired
+            let pk_id = Uuid::new_v4();
+            let primary_key_index = SqlTuple(vec![
+                Some(BaseSqlTypes::Uuid(pk_id)),
+                Some(BaseSqlTypes::Uuid(table_id)),
+                Some(BaseSqlTypes::Text(format!(
+                    "{}_primary_key_index",
+                    create_table.table_name
+                ))),
+                Some(BaseSqlTypes::Array(primary_key_cols)),
+                Some(BaseSqlTypes::Bool(true)),
+            ]);
+            let pg_index = SystemTables::PgIndex.value();
+            self.cons_man
+                .clone()
+                .insert_row(tran_id, &pg_index, primary_key_index)
+                .await?;
+
+            //Now we can insert the constraint
+            let primary_key_constraint = SqlTuple(vec![
+                Some(BaseSqlTypes::Uuid(Uuid::new_v4())),
+                Some(BaseSqlTypes::Uuid(table_id)),
+                Some(BaseSqlTypes::Uuid(pk_id)),
+                Some(BaseSqlTypes::Text(format!(
+                    "{}_primary_key",
+                    create_table.table_name
+                ))),
+                Some(BaseSqlTypes::Text(ConstraintMapper::PrimaryKey.to_string())),
+            ]);
+            let pg_constraint = SystemTables::PgConstraint.value();
+            self.cons_man
+                .clone()
+                .insert_row(tran_id, &pg_constraint, primary_key_constraint)
                 .await?;
         }
+
         Ok(vec![])
     }
 }
