@@ -1,3 +1,5 @@
+pub mod block_layer;
+
 mod constraint_manager;
 pub use constraint_manager::ConstraintManager;
 pub use constraint_manager::ConstraintManagerError;
@@ -12,14 +14,7 @@ mod index_formats;
 mod index_manager;
 pub use index_manager::IndexManager;
 
-mod file_manager;
-pub use file_manager::FileManager;
-pub use file_manager::FileManagerError;
-
-mod lock_manager;
-//pub use lock_manager::LockManager;
-
-mod page_formats;
+pub mod page_formats;
 
 pub mod row_formats;
 
