@@ -1,6 +1,11 @@
 mod attribute;
 pub use attribute::Attribute;
 
+mod constraints;
+pub use constraints::Constraint;
+pub use constraints::ConstraintMapper;
+pub use constraints::PrimaryKeyConstraint;
+
 mod index;
 pub use index::Index;
 
